/*
qr_tool on newlib
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stddef.h>
#include <kos32sys.h>
#include "kolibri_colors.h"
#include "kolibri_libimg.h"

#include <quirc.h>

#define NEW_IMG_H 256 
#define NEW_IMG_W 256

//kolibri_system_colors sys_color_table;  // system colors table

/* -------------------------------------------------------------------- */

char* load_img(char* fname, uint32_t* read_sz) {
    FILE *f = fopen(fname, "rb");
    if (!f) {
        printf("Can't open file: %s\n", fname);
        exit(0);
    }
    if (fseek(f, 0, SEEK_END)) {
        printf("Can't SEEK_END file: %s\n", fname);
        exit(0);
    }
    int filesize = ftell(f);
    rewind(f);
    char* fdata = malloc(filesize);
    if(!fdata) {
        printf("No memory for file %s\n", fname);
        exit(0);
    }
    *read_sz = fread(fdata, 1, filesize, f);
    if (ferror(f)) {
        printf("Error reading file %s\n", fname);
        exit(0);
    }
    fclose(f);
    return fdata;
}

void get_rgb(Image *img, int x, int y, uint8_t *r, uint8_t *g, uint8_t *b) {
    int offset = (y*img->Width + x)*3;
    uint8_t *map = (uint8_t*)img->Data;
    *r = *(map + offset + 2);
    *g = *(map + offset + 1);
    *b = *(map + offset);
}

int main(int argc, char *argv[]) {
    if (kolibri_libimg_init() == -1){
        printf("Error loading lib_img.obj\n");
        exit(-1);
    }

    //kolibri_get_system_colors(&sys_color_table); // Get system colors theme
    //set_wanted_events_mask(0xC0000027);

    char *file_path;
    if (argc < 2) {
        printf("usage: %s <file>\n", strrchr(argv[0], '/') + 1);
        exit(-1);
    } else {
        file_path = argv[1];
    }

    uint32_t img_size; 
    void *file_data = load_img(file_path, &img_size); // Get RAW data and size 
    
    Image* image = img_decode(file_data, img_size, 0); // Decode RAW data to Image data
    uint32_t image_orig_width = image->Width;
    uint32_t image_orig_height = image->Height;
    printf("original: image->Width = %d, Image->Height = %d, original image type = %d\n\n", image_orig_width, image_orig_height, image->Type);    
    
    if (image->Type != IMAGE_BPP24) { 
        image = img_convert(image, NULL, IMAGE_BPP24, 0, 0); // Convert image to RGB 24
        if (!image) {
            puts("Сonvert to BPP24 error!: \n");  
            exit(-1);
        }
    }

    if (image->Type != IMAGE_BPP8g) { 
        image = img_convert(image, NULL, IMAGE_BPP8g, 0, 0); // Convert image to grayscale
        if (!image) {
            puts("Сonvert to BPP8g error!: \n");  
            exit(-1);
        }
    }

    //img_destroy(image);  // Destroy image structure
    free(file_data);  // Free allocated file_data buffer 

    /* -------------------------------------------------------------------- */
    puts("Starting QUIRC...\n");
    struct quirc *qr;

    qr = quirc_new();
    if (!qr) {
        puts("Failed to allocate memory");
        exit(-1);
    }

    if (quirc_resize(qr, image_orig_width, image_orig_height) < 0) {
        puts("Failed to allocate video memory");
        exit(-1);
    }

    void *gsbuf = image->Data;
    uint32_t gsbuf_size = image_orig_width*image_orig_height;

    uint8_t *buffer;
    int w, h;
    puts("quirc_begin()...\n");
    buffer = quirc_begin(qr, &w, &h);
    //printf("buffer = %x  qr = %x\n", buffer, qr);
    //printf("w = %d   h = %d\n", w, h);
    memcpy(buffer, gsbuf, gsbuf_size);
    puts("quirc_end()...\n");
    quirc_end(qr);

    
    int num_codes, i;
    puts("quirc_count()...\n");
    num_codes = quirc_count(qr);
    printf("    NUM_CODES = %d\n", num_codes);
    for (i = 0; i < num_codes; i++) {
        struct quirc_code code;
        struct quirc_data data;
        quirc_decode_error_t err;

        quirc_extract(qr, i, &code);

        // decoding stage
        err = quirc_decode(&code, &data);
        if (err) {
            printf("    DECODE FAILED: %s\n", quirc_strerror(err));
        } else {
            printf("\n    RECOGNIZED DATA: %s\n", data.payload);
        }
    }
    
    puts("\nquirc_destroy()...\n");
    quirc_destroy(qr);

    img_destroy(image);  // Destroy image structure

    return 0;
}
